@@ -1,12 +1,12 @@
 #
-# Revelation 0.3.4 - a password manager for GNOME 2
+# Revelation 0.4.3 - a password manager for GNOME 2
 # http://oss.codepoet.no/revelation/
-# $Id: __init__.py 153 2004-09-27 17:29:40Z erikg $
+# $Id: __init__.py 322 2005-03-31 17:43:29Z erikg $
 #
 # Library initialization script
 #
 #
-# Copyright (c) 2003-2004 Erik Grinaker
+# Copyright (c) 2003-2005 Erik Grinaker
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -23,19 +23,17 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #
 
+import crack
+import util
+import config
+import authmanager
+import datahandler
+import io
+import entry
+import data
+import ui
+import dialog
 
-import datahandler, misc, stock, entry, widget, data, io, dialog, sys
-
-APPNAME = "Revelation"
-VERSION = "0.3.4"
-DATAVERSION = 1
-RELNAME = "Cellardoor"
-URL = "http://oss.codepoet.no/revelation/"
-AUTHOR = "Erik Grinaker <erikg@codepoet.no>"
-COPYRIGHT = "Copyright \302\251 2003-2004 Erik Grinaker"
-
-PREFIX = sys.prefix
-DATADIR = PREFIX + "/share/revelation"
 
 # set up some exceptions
 class Error(Exception):
