@@ -1,12 +1,12 @@
 #
-# Revelation 0.3.4 - a password manager for GNOME 2
+# Revelation 0.4.3 - a password manager for GNOME 2
 # http://oss.codepoet.no/revelation/
-# $Id: __init__.py 153 2004-09-27 17:29:40Z erikg $
+# $Id: __init__.py 322 2005-03-31 17:43:29Z erikg $
 #
 # Module for data handlers
 #
 #
-# Copyright (c) 2003-2004 Erik Grinaker
+# Copyright (c) 2003-2005 Erik Grinaker
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -23,23 +23,41 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #
 
-from base import *
-from fpm import *
-from gpass import *
-from netrc import *
-from rvl import *
-from xhtml import *
+from base import HandlerError, DataError, FormatError, PasswordError, VersionError
+from fpm import FPM
+from gpass import GPass
+from netrc import NetRC
+from rvl import RevelationXML, Revelation
+from text import PlainText
+from xhtml import XHTML
 
 HANDLERS = [
 	FPM,
 	GPass,
 	NetRC,
+	PlainText,
 	Revelation,
 	XHTML,
 	RevelationXML
 ]
 
 
+class DetectError(Exception):
+	"Exception for autodetection error"
+	pass
+
+
+def detect_handler(input):
+	"Detects which handler may process a data stream"
+
+	for handler in get_import_handlers():
+		if handler().detect(input) == True:
+			return handler
+
+	else:
+		raise DetectError
+
+
 def get_export_handlers():
 	"Returns a list of handlers which can export"
 
