@@ -1,12 +1,12 @@
 #
-# Revelation 0.3.4 - a password manager for GNOME 2
+# Revelation 0.4.3 - a password manager for GNOME 2
 # http://oss.codepoet.no/revelation/
-# $Id: entry.py 153 2004-09-27 17:29:40Z erikg $
+# $Id: entry.py 322 2005-03-31 17:43:29Z erikg $
 #
 # Module containing entry information
 #
 #
-# Copyright (c) 2003-2004 Erik Grinaker
+# Copyright (c) 2003-2005 Erik Grinaker
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -23,42 +23,31 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #
 
-import revelation, gobject, gtk, time
+from revelation import ui
+
+import copy, time
 
 
+DATATYPE_FILE		= "file"
 DATATYPE_EMAIL		= "email"
 DATATYPE_PASSWORD	= "password"
-DATATYPE_TEXT		= "text"
+DATATYPE_STRING		= "string"
 DATATYPE_URL		= "url"
 
 
 
-class EntryError(Exception):
+class EntryFieldError(Exception):
+	"Exception for invalid entry fields"
 	pass
 
-class EntryFieldError(EntryError):
-	pass
 
-class EntryTypeError(EntryError):
+class EntryTypeError(Exception):
+	"Exception for invalid entry types"
 	pass
 
 
 
-class LaunchError(Exception):
-	pass
-
-class NoLaunchError(LaunchError):
-	pass
-
-class LaunchDataError(LaunchError):
-	pass
-
-class LaunchFormatError(LaunchError):
-	pass
-
-
-
-class Entry(gobject.GObject):
+class Entry(object):
 	"An entry object"
 
 	id		= None
@@ -66,40 +55,24 @@ class Entry(gobject.GObject):
 	icon		= None
 
 	def __init__(self):
-		gobject.GObject.__init__(self)
-
 		self.name		= ""
 		self.description	= ""
-		self.updated		= 0
+		self.updated		= int(time.time())
 		self.fields		= []
 
 
-	def can_launch(self):
-		"Checks if the entry can be launched"
+	def __getitem__(self, key):
+		return self.get_field(key).value
 
-		try:
-			self.get_launcher()
 
-		except NoLaunchError:
-			return gtk.FALSE
-
-		except LaunchError:
-			return gtk.TRUE
-
-		else:
-			return gtk.TRUE
+	def __setitem__(self, key, value):
+		self.get_field(key).value = value
 
 
 	def copy(self):
 		"Create a copy of the entry"
 
-		entry = type(self)()
-		entry.name = self.name
-		entry.description = self.description
-		entry.updated = self.updated
-		entry.fields = self.fields[:]
-
-		return entry
+		return copy.deepcopy(self)
 
 
 	def get_field(self, fieldtype):
@@ -113,94 +86,29 @@ class Entry(gobject.GObject):
 			raise EntryFieldError
 
 
-	def get_launcher(self):
-		"Returns the launcher for the entry"
-
-		try:
-			launcher = revelation.data.config_get("launcher/" + self.id)
-
-			if launcher in [ "", None ]:
-				raise NoLaunchError
-
-		except revelation.data.ConfigError:
-			raise NoLaunchError
-
-
-		return launcher
-
-
-	def get_launcher_parsed(self):
-		"Returns the parsed launcher for an entry"
-
-		launcher = self.get_launcher()
-
-		subst = {}
-
-		for field in self.fields:
-			subst[field.symbol] = field.value
-
-		try:
-			command = revelation.misc.parse_subst(launcher, subst)
-
-		except revelation.misc.SubstValueError:
-			raise LaunchDataError
-
-		except revelation.misc.SubstFormatError:
-			raise LaunchFormatError
-
-		return command
-
-
-	def get_updated_age(self):
-		"Get the age of an entry"
-
-		return revelation.misc.timediff_simple(self.updated)
-
-
-	def get_updated_iso(self):
-		"Get the update time in ISO format"
-
-		return time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(self.updated))
-
-
 	def has_field(self, fieldtype):
-		"Checks if the entry has a particular field"
+		"Check if the entry has a field"
 
-		for field in self.fields:
-			if type(field) == fieldtype:
-				return gtk.TRUE
+		try:
+			self.get_field(fieldtype)
+			return True
 
-		else:
-			return gtk.FALSE
+		except EntryFieldError:
+			return False
 
 
-	def import_data(self, entry):
-		"Imports data from a different entry"
+	def mirror(self, entry):
+		"Makes this entry mirror a different entry (same data)"
+
+		if type(self) != type(entry):
+			raise EntryTypeError
 
 		self.name		= entry.name
 		self.description	= entry.description
 		self.updated		= entry.updated
 
 		for field in entry.fields:
-			if self.has_field(type(field)):
-				self.get_field(type(field)).value = field.value
-
-
-	def launch(self):
-		"Attempts to launch the entry"
-
-		revelation.io.execute_child(self.get_launcher_parsed())
-
-
-	def lookup_field(self, id):
-		"Looks up a field based on an id"
-
-		for field in self.fields:
-			if field.id == id:
-				return field
-
-		else:
-			raise EntryFieldError
+			self[type(field)] = field.value
 
 
 
@@ -208,7 +116,8 @@ class FolderEntry(Entry):
 
 	id		= "folder"
 	typename	= "Folder"
-	icon		= revelation.stock.STOCK_FOLDER
+	icon		= ui.STOCK_ENTRY_FOLDER
+	openicon	= ui.STOCK_ENTRY_FOLDER_OPEN
 
 	def __init__(self):
 		Entry.__init__(self)
@@ -219,7 +128,7 @@ class CreditcardEntry(Entry):
 
 	id		= "creditcard"
 	typename	= "Creditcard"
-	icon		= revelation.stock.STOCK_ACCOUNT_CREDITCARD
+	icon		= ui.STOCK_ENTRY_CREDITCARD
 
 	def __init__(self):
 		Entry.__init__(self)
@@ -238,7 +147,7 @@ class CryptoKeyEntry(Entry):
 
 	id		= "cryptokey"
 	typename	= "Crypto Key"
-	icon		= revelation.stock.STOCK_ACCOUNT_CRYPTOKEY
+	icon		= ui.STOCK_ENTRY_CRYPTOKEY
 
 	def __init__(self):
 		Entry.__init__(self)
@@ -256,7 +165,7 @@ class DatabaseEntry(Entry):
 
 	id		= "database"
 	typename	= "Database"
-	icon		= revelation.stock.STOCK_ACCOUNT_DATABASE
+	icon		= ui.STOCK_ENTRY_DATABASE
 
 	def __init__(self):
 		Entry.__init__(self)
@@ -274,7 +183,7 @@ class DoorEntry(Entry):
 
 	id		= "door"
 	typename	= "Door lock"
-	icon		= revelation.stock.STOCK_ACCOUNT_DOOR
+	icon		= ui.STOCK_ENTRY_DOOR
 
 	def __init__(self):
 		Entry.__init__(self)
@@ -290,7 +199,7 @@ class EmailEntry(Entry):
 
 	id		= "email"
 	typename	= "Email"
-	icon		= revelation.stock.STOCK_ACCOUNT_EMAIL
+	icon		= ui.STOCK_ENTRY_EMAIL
 
 	def __init__(self):
 		Entry.__init__(self)
@@ -308,7 +217,7 @@ class FTPEntry(Entry):
 
 	id		= "ftp"
 	typename	= "FTP"
-	icon		= revelation.stock.STOCK_ACCOUNT_FTP
+	icon		= ui.STOCK_ENTRY_FTP
 
 	def __init__(self):
 		Entry.__init__(self)
@@ -326,7 +235,7 @@ class GenericEntry(Entry):
 
 	id		= "generic"
 	typename	= "Generic"
-	icon		= revelation.stock.STOCK_ACCOUNT_GENERIC
+	icon		= ui.STOCK_ENTRY_GENERIC
 
 	def __init__(self):
 		Entry.__init__(self)
@@ -343,7 +252,7 @@ class PhoneEntry(Entry):
 
 	id		= "phone"
 	typename	= "Phone"
-	icon		= revelation.stock.STOCK_ACCOUNT_PHONE
+	icon		= ui.STOCK_ENTRY_PHONE
 
 	def __init__(self):
 		Entry.__init__(self)
@@ -359,7 +268,7 @@ class ShellEntry(Entry):
 
 	id		= "shell"
 	typename	= "Shell"
-	icon		= revelation.stock.STOCK_ACCOUNT_SHELL
+	icon		= ui.STOCK_ENTRY_SHELL
 
 	def __init__(self):
 		Entry.__init__(self)
@@ -377,7 +286,7 @@ class WebEntry(Entry):
 
 	id		= "website"
 	typename	= "Website"
-	icon		= revelation.stock.STOCK_ACCOUNT_WEBSITE
+	icon		= ui.STOCK_ENTRY_WEBSITE
 
 	def __init__(self):
 		Entry.__init__(self)
@@ -390,7 +299,23 @@ class WebEntry(Entry):
 
 
 
-class Field(gobject.GObject):
+ENTRYLIST = [
+	FolderEntry,
+	CreditcardEntry,
+	CryptoKeyEntry,
+	DatabaseEntry,
+	DoorEntry,
+	EmailEntry,
+	FTPEntry,
+	GenericEntry,
+	PhoneEntry,
+	ShellEntry,
+	WebEntry
+]
+
+
+
+class Field(object):
 	"An entry field object"
 
 	id		= None
@@ -402,45 +327,11 @@ class Field(gobject.GObject):
 	value		= None
 
 	def __init__(self, value = ""):
-		gobject.GObject.__init__(self)
-
 		self.value = value
 
 
-	def generate_display_widget(self):
-		"Generates a widget for displaying the field"
-
-		if self.datatype == DATATYPE_EMAIL:
-			widget = revelation.widget.HRef("mailto:" + self.value, revelation.misc.escape_markup(self.value))
-
-		elif self.datatype == DATATYPE_PASSWORD:
-			widget = revelation.widget.PasswordLabel(revelation.misc.escape_markup(self.value))
-
-		elif self.datatype == DATATYPE_URL:
-			widget = revelation.widget.HRef(self.value, revelation.misc.escape_markup(self.value))
-
-		else:
-			widget = revelation.widget.Label(revelation.misc.escape_markup(self.value))
-			widget.set_selectable(gtk.TRUE)
-
-		return widget
-
-
-	def generate_edit_widget(self):
-		"Generates a widget for editing the field"
-
-		if type(self) == PasswordField:
-			entry = revelation.widget.PasswordEntryGenerate()
-
-		elif self.datatype == DATATYPE_PASSWORD:
-			entry = revelation.widget.PasswordEntry()
-
-		else:
-			entry = revelation.widget.Entry()
-
-		entry.set_text(self.value)
-
-		return entry
+	def __str__(self):
+		return self.value is not None and self.value or ""
 
 
 
@@ -450,7 +341,7 @@ class CardnumberField(Field):
 	name		= "Card number"
 	description	= "The number of a creditcard, usually a 16-digit number"
 	symbol		= "N"
-	datatype	= DATATYPE_TEXT
+	datatype	= DATATYPE_STRING
 
 
 
@@ -460,7 +351,7 @@ class CardtypeField(Field):
 	name		= "Card type"
 	description	= "The type of creditcard, like MasterCard or VISA"
 	symbol		= "C"
-	datatype	= DATATYPE_TEXT
+	datatype	= DATATYPE_STRING
 
 
 
@@ -470,7 +361,7 @@ class CCVField(Field):
 	name		= "CCV number"
 	description	= "A Credit Card Verification number, normally a 3-digit code found on the back of a card"
 	symbol		= "V"
-	datatype	= DATATYPE_TEXT
+	datatype	= DATATYPE_STRING
 
 
 
@@ -480,7 +371,7 @@ class CertificateField(Field):
 	name		= "Certificate"
 	description	= "A certificate, such as an X.509 SSL Certificate"
 	symbol		= "x"
-	datatype	= DATATYPE_TEXT
+	datatype	= DATATYPE_FILE
 
 
 
@@ -500,7 +391,7 @@ class DatabaseField(Field):
 	name		= "Database"
 	description	= "A database name"
 	symbol		= "D"
-	datatype	= DATATYPE_TEXT
+	datatype	= DATATYPE_STRING
 
 
 
@@ -510,7 +401,7 @@ class DomainField(Field):
 	name		= "Domain"
 	description	= "An Internet or logon domain, like amazon.com or a Windows logon domain"
 	symbol		= "d"
-	datatype	= DATATYPE_TEXT
+	datatype	= DATATYPE_STRING
 
 
 
@@ -530,7 +421,7 @@ class ExpirydateField(Field):
 	name		= "Expiry date"
 	description	= "The month that the credit card validity expires"
 	symbol		= "E"
-	datatype	= DATATYPE_TEXT
+	datatype	= DATATYPE_STRING
 
 
 
@@ -540,7 +431,7 @@ class HostnameField(Field):
 	name		= "Hostname"
 	description	= "The name of a computer, like computer.domain.com or MYCOMPUTER"
 	symbol		= "h"
-	datatype	= DATATYPE_TEXT
+	datatype	= DATATYPE_STRING
 
 
 
@@ -550,7 +441,7 @@ class KeyfileField(Field):
 	name		= "Key File"
 	description	= "A key file, used for authentication for example via ssh or to encrypt X.509 certificates"
 	symbol		= "f"
-	datatype	= DATATYPE_TEXT
+	datatype	= DATATYPE_FILE
 
 
 
@@ -560,7 +451,7 @@ class LocationField(Field):
 	name		= "Location"
 	description	= "A physical location, like office entrance"
 	symbol		= "L"
-	datatype	= DATATYPE_TEXT
+	datatype	= DATATYPE_STRING
 
 
 
@@ -580,7 +471,7 @@ class PhonenumberField(Field):
 	name		= "Phone number"
 	description	= "A telephone number"
 	symbol		= "n"
-	datatype	= DATATYPE_TEXT
+	datatype	= DATATYPE_STRING
 
 
 
@@ -600,7 +491,7 @@ class PortField(Field):
 	name		= "Port number"
 	description	= "A network port number, used to access network services directly"
 	symbol		= "o"
-	datatype	= DATATYPE_TEXT
+	datatype	= DATATYPE_STRING
 
 
 
@@ -620,89 +511,73 @@ class UsernameField(Field):
 	name		= "Username"
 	description	= "A name or other identification used to identify yourself"
 	symbol		= "u"
-	datatype	= DATATYPE_TEXT
+	datatype	= DATATYPE_STRING
 
 
 
+FIELDLIST = [
+	CardnumberField,
+	CardtypeField,
+	CCVField,
+	CertificateField,
+	CodeField,
+	DatabaseField,
+	DomainField,
+	EmailField,
+	ExpirydateField,
+	HostnameField,
+	KeyfileField,
+	LocationField,
+	PasswordField,
+	PhonenumberField,
+	PINField,
+	PortField,
+	URLField,
+	UsernameField
+]
+
+
 
 def convert_entry_generic(entry):
-	"Converts to a generic account, tries to keep as much data as possible"
+	"Converts an entry to GenericEntry, tries to keep as much data as possible"
 
-	# set up the initial generic entry
 	generic = GenericEntry()
-	generic.import_data(entry)
+	generic.name = entry.name
+	generic.description = entry.description
+	generic.updated = entry.updated
 
 	# do direct field copies
 	for field in generic.fields:
 		if entry.has_field(type(field)):
-			field.value = entry.get_field(type(field)).value
-
+			field.value = entry[type(field)]
 
 	# handle special conversions
-	field_hostname = generic.get_field(HostnameField)
-	field_username = generic.get_field(UsernameField)
-	field_password = generic.get_field(PasswordField)
-
 	if type(entry) == CreditcardEntry:
-		field_username.value = entry.get_field(CardnumberField).value
-		field_password.value = entry.get_field(PINField).value
+		generic[UsernameField] = entry[CardnumberField]
+		generic[PasswordField] = entry[PINField]
 
 	elif type(entry) == CryptoKeyEntry:
-		field_username.value = entry.get_field(KeyfileField).value
+		generic[UsernameField] = entry[KeyfileField]
 
 	elif type(entry) == DatabaseEntry:
-		if entry.get_field(DatabaseField).value != "":
-			field_hostname.value = entry.get_field(DatabaseField).value + "@" + field_hostname.value
+		if entry[DatabaseField] != "":
+			generic[HostnameField] = entry[DatabaseField] + "@" + entry[HostnameField]
 
 	elif type(entry) == DoorEntry:
-		field_password.value = entry.get_field(CodeField).value
-		field_hostname.value = entry.get_field(LocationField).value
+		generic[PasswordField] = entry[CodeField]
+		generic[HostnameField] = entry[LocationField]
 
 	elif type(entry) == FTPEntry:
-		
-		field_hostname.value = "ftp://" + entry.get_field(HostnameField).value
-
-		if entry.get_field(PortField).value != "":
-			field_hostname.value += ":" + entry.get_field(PortField).value
+		if entry[PortField] not in ( "", "21" ):
+			generic[HostnameField] += ":" + entry[PortField]
 
 	elif type(entry) == PhoneEntry:
-		field_username.value = entry.get_field(PhonenumberField).value
-		field_password.value = entry.get_field(PINField).value
+		generic[UsernameField] = entry[PhonenumberField]
+		generic[PasswordField] = entry[PINField]
 
 	elif type(entry) == WebEntry:
-		field_hostname.value  = entry.get_field(URLField).value
+		generic[HostnameField] = entry[URLField]
 
 
 	return generic
 
-
-
-def get_entry_list():
-	"Returns a sorted list of all available entry types"
-
-	return [
-		FolderEntry,
-		CreditcardEntry,
-		CryptoKeyEntry,
-		DatabaseEntry,
-		DoorEntry,
-		EmailEntry,
-		FTPEntry,
-		GenericEntry,
-		PhoneEntry,
-		ShellEntry,
-		WebEntry
-	]
-
-
-
-def lookup_entry(id):
-	"Looks up an entry based on an id"
-
-	for entry in get_entry_list():
-		if entry.id == id:
-			return entry
-
-	else:
-		return None
-
